# About
AIRTLS Backend Mock is a library and shell tools for communication with Anchor device over USB, or BLE.

# Requirements

- Python 3.8 (or newer)
- OS: Linux (tested on Ubuntu 20.04), Windows (tested on Windows 10)

# How To

1. Install requirements  
`pip3 install -r ./requirements.txt`

2. Start shell  
`python3 ./software/app_shell/main.py`

3. Use command `help` to print help message

4. Check if device was found. Program will print all devices and available interfaces.  
Example output:
```
DID: AF319E
  - AIRTLS-BLE  FD:5D:4B:AF:31:9E       b'Hello AIRTLS'

DID: FB0CD1
  - AIRTLS-BLE  EA:23:69:FB:0C:D1       b'Hello AIRTLS'
  - AIRTLS-USB-TRACE    /dev/ttyACM2
  - AIRTLS-USB-SHELL    /dev/ttyACM1
  - AIRTLS-USB-PROTOCOL /dev/ttyACM0

```

5. Connect to the device  (use tab for autocompletion):  
`connect FB0CD1:AIRTLS-USB-PROTOCOL`  
Device format:  
`<DID>:<INTERFACE>`  
*DID* - hex string (6 characters)  
*INTERFACE* - Interface name. Supported interfaces:     
    - `AIRTLS-USB-PROTOCOL`
    - `AIRTLS-BLE`

6. Program will periodically print number of collected frames and interface speed
   NOTE: By default DEBUG logs are disabled (`software/lib/__init__.py`)
```
[2023-08-02 09:37:03,241] | lib.interface.abstract_interface | DEBUG | Interface Speed: 389.66 B/s
[2023-08-02 09:37:03,242] | lib.device | DEBUG | Messages in queue: 51
```

7. Print received frames with command:  
`dump-frames`  
Example output (each chunk represents single packet):
```
RX Timestamp: 2023-08-01 15:19:24.983643
Data:{'header': {'type': 1, 'length': 36}, 'value': {'a2a': {'tof': [300, 302], 'did': '14283c', 'rssi_received_by_responder': 5, 'rssi_received_by_transmitter': 12}, 'tag_lists': [{'tag_count': 2, 'number_of_tofs': 3, 'precision': 0, 'tags': [{'did': '212223', 'rssi_received_by_responder': 10, 'rssi_received_by_transmitter': 12, 'tof': [200, 201, 202]}, {'did': '212224', 'rssi_received_by_responder': 11, 'rssi_received_by_transmitter': 13, 'tof': [204, 205, 206]}]}], 'did': '050a0f', 'cycle_id': 5000, 'tag_lists_count': 1}}

RX Timestamp: 2023-08-01 15:19:25.083920
Data:{'header': {'type': 1, 'length': 36}, 'value': {'a2a': {'tof': [300, 302], 'did': '14283c', 'rssi_received_by_responder': 5, 'rssi_received_by_transmitter': 12}, 'tag_lists': [{'tag_count': 2, 'number_of_tofs': 3, 'precision': 0, 'tags': [{'did': '212223', 'rssi_received_by_responder': 10, 'rssi_received_by_transmitter': 12, 'tof': [200, 201, 202]}, {'did': '212224', 'rssi_received_by_responder': 11, 'rssi_received_by_transmitter': 13, 'tof': [204, 205, 206]}]}], 'did': '050a0f', 'cycle_id': 5000, 'tag_lists_count': 1}}
```


8. Enable "live data" logging with command:  
`logging-enable`  
Disable "live data" logging with command:  
`logging-disable`

9. Disconnect from the device with command:  
`disconnect`

10. Exit the application with `exit` command.  
    NOTE: If you brutally kill the application, BLE connection will remain and Anchor device must be restarted.


# APP Socket receiver

Receives frames from UDP socket and prints them as JSON
## Input arguments:
#### '-t', '--type' - optional ACP TLV frame type filter.

Where type:  
 1 :  TimeOfFlightFrame,  
 2 :  SerialDataFrame,  
 3 :  AdminDistributionInfoFrame,  
 4 :  CompressedDataFrame,  
 5 :  AccelerometerDataFrame,  
 6 :  GyroscopeDataFrame,  
 7 :  MagnetometerFrame,  
 8 :  ImpactFrame,  
      
Example - print only accelerometer data frames:
```
python3 ./software/app_socket_receiver/main.py -t 5
```


#### '-d', '--did' - optional device ID filter.

Example - print only frames from device 'c2e151':
```
python3 ./software/app_socket_receiver/main.py -d c2e151
``````


#### Example output:
```
Frame type counters: {1: 43, 3: 30, 5: 43, 6: 43, 7: 43, 8: 43}
DID frame counters: {'fb0cd1': 56, 'c2e151': 189}
{
    "header": {
        "type": 8,
        "length": 9
    },
    "value": {
        "did": "c2e151",
        "cycle_id": 29448,
        "slot_id": 55,
        "hit_type": 1,
        "hit_position": 127,
        "hit_vector": 1070
    }
}
```