#!/usr/bin/env python3
# -----------------------------------------------------------------------------
# File:     main.py
# Author:   Daniel Dyringer
# Brief:    AIRTLS Anchor Demo App
# -----------------------------------------------------------------------------

import time
import signal
import sys
import os

sys.path.append(os.path.join(os.path.dirname(__file__), ".."))  # noqa

from lib_acp_tlv import *

DID = "C99887"
# INTERFACE_TYPE = "AIRTLS-BLE"
INTERFACE_TYPE = "AIRTLS-USB-PROTOCOL"

def main():
    device_manager = DeviceManager()
    device_manager.enable_logging(True)
    device_manager.get_all_devices()

    device: InterfaceDevice = device_manager.get_device(DID, INTERFACE_TYPE)
    assert device != None

    device.connect()

    while True:
        time.sleep(1)

        data = {
                "anchors": [
                    {
                        "did": DID,
                        "pattern": 20,
                        "brightness" : 1,
                        "cycle_id": 0,
                    },
                    {
                        "did": "000000",
                        "pattern": 0,
                        "brightness" : 0,
                        "cycle_id": 0,
                    },
                ]
            }

        frame = protocol_helper_createBinaryFrame(CommandBlinkFrame, data)
        device.transmit(frame)

if __name__ == '__main__':
    signal.signal(signal.SIGINT, signal.SIG_DFL)
    main()