
import logging
logger = logging.getLogger(__name__)
import traceback

class BitStichUnpacker:
    map = {
        '0' : '0000',
        '1' : '0001',
        '2' : '0010',
        '3' : '0011',
        '4' : '0100',
        '5' : '0101',
        '6' : '0110',
        '7' : '0111',
        '8' : '1000',
        '9' : '1001',
        'a' : '1010',
        'b' : '1011',
        'c' : '1100',
        'd' : '1101',
        'e' : '1110',
        'f' : '1111'
    }

    @staticmethod
    def _convert(hex : str):
        out = ""
        for i in range(0, len(hex), 2):
            out = out  + (BitStichUnpacker.map[hex[i]] + BitStichUnpacker.map[hex[i+1]])[::-1]
        return out

    def unpack_samples(data: bytearray, bits_per_value):
        accelerometer_samples = []
        try:
            hex_data = data.hex()
            bits = BitStichUnpacker._convert(hex_data)
            accelerometer_samples = []
            samples_count = int(len(bits) / bits_per_value)
            for sample_index in range(samples_count):
                bit_index = int(sample_index * bits_per_value)
                sample_bits: str = bits[bit_index : (bit_index + bits_per_value)][::-1]
                if sample_bits[0] == '1':
                    sample_bits2 = '0b' + '1' * (32-bits_per_value) + sample_bits
                else:
                    sample_bits2 = '0b' + '0' * (32-bits_per_value) + sample_bits

                sample_raw = int(sample_bits2, base=2)
                sample_bytes = sample_raw.to_bytes(4, "little", signed=False)
                sample = int.from_bytes(sample_bytes, "little", signed=True)
                accelerometer_samples.append(sample)
        except Exception as e:
            logger.error(e)
            logger.error(traceback.format_exc())
        
        return accelerometer_samples